# RevitDoc--提取数据格式
点信息格式为：
```json
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
}
```
小数点后保留两位，单位为公制 ；
OutLine为点集合，围成闭合区域，注意将起始点添加到集合中
## 项目
输入：Revit模型文件地址 *\F1.rvt
输出：
```json
{
    "MBIName":"PjXXX_001",
    "PlanName":"F1-saga",
    "Parameters":[

    ],
    "Elements":[

    ]
}
```

| 名称   | 解释   | 示例    
|:---:|:---:|:---:|
| MBIName   | BIM模型中存储的对应物理世界的文件序号   | PjXXX_001   | 
| PlanName   | 当前楼层代表的平面图名称   | F1-saga   | 
| Parameters   | 从BIM模型中读取到的参数，目前值为空   | 格式参见参数定义   | 
| Elements   | BIM模型中包含的构件  类型的分组：Units,Levels,Grids,Walls,VirtualWalls,Columns,Spaces,Doors,Windows,Equipments,EquipParts,Ducts,Pipes,Connectors,BoundarySegments   | 参见构件的定义；     | 

### 
## 项目单位
输入：
输出：

```json
{
            "Id":"10005",
            "ElementType":"Unit",
            "SourceId":"",
            "Name":"Length",
            "Value":"mm",
            "Tag":"长度单位"
        }
        
```  

| 名称   | 解释   | 示例    
|:---:|:---:|:---:|
| Id   | 在提取过程中生成的Id   | 10001   | 
| ElementType   | 构件类型，这里标识为标上单位   | Unit   | 
| SourceId   | 对应BIM模型中的Id   |    | 
| Name   | 单位名称   | Length   | 
| Value   | 单位的值   | mm   | 
| Tag   | 单位描述   | 长度单位   | 


## 标高
输入：
输出：
```json
{
            "Id":"10005",
            "ElementType":"Level",
            "SourceId":"311",
            "Name":"标高 1",
            "Value":0,
           
        }
```


| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10001   | 
| ElementType   | 构件类型，这里标识为标高   | Level   | 
| SourceId   | 对应BIM模型中的Id   | 311   | 
| Name   | 标高名称   | 标高 1   | 
| Value   | 标高的高度   | 0   | 

### 
## 轴网
输入：
输出：
```json
{
                "Location":{
                    "Type":"Line",
                    "Points":[
                        {
                            "X":16770.65,
                            "Y":7008.54,
                            "Z":0
                        },
                        {
                            "X":-14798.19,
                            "Y":7008.54,
                            "Z":0
                        }
                    ]
                },
                "OutLine":[

                ],
                "SourceId":"359378",
                "Name":"1",
                "Id":"10067",
                "ElementType":"Grid"
            }
```

| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10001   | 
| ElementType   | 构件类型，这里标识为轴网   | Grid   | 
| SourceId   | 对应BIM模型中的Id   |    | 
| Name   | 轴网名称   | A,B,C,...1,2,3,...   | 
| Location   | 定位信息   | 格式见定位信息   | 
| Description   | 描述   |    | 

## 构件-墙
注意：有门窗的墙，墙轮廓为底部，多个拆分后的轮廓
```json
{
                "Width":200,
                "Location":{
                    "Type":"Line",
                    "Points":[
                        {
                            "X":-2298.19,
                            "Y":7008.54,
                            "Z":0
                        },
                        {
                            "X":618.94,
                            "Y":-4691.46,
                            "Z":0
                        }
                    ]
                },
                "OutLine":[
                    [
                        {
                            "X":-2376.32,
                            "Y":6908.54,
                            "Z":0
                        },
                        {
                            "X":521.91,
                            "Y":-4715.65,
                            "Z":0
                        },
                        {
                            "X":715.97,
                            "Y":-4667.27,
                            "Z":0
                        },
                        {
                            "X":-2170.2,
                            "Y":6908.54,
                            "Z":0
                        },
                        {
                            "X":-2376.32,
                            "Y":6908.54,
                            "Z":0
                        }
                    ]
                ],
                "SourceId":"313107",
                "Name":"常规 - 200mm",
                "Id":"10005",
                "ElementType":"Wall"
            }
```

| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10009   | 
| ElementType   | 构件类型，这里标识为墙   | Wall   | 
| SourceId   | 对应BIM模型中的Id   | 1159597   | 
| Name   | 构件名称   |    | 
| Location   | 定位信息   | 线定位信息   | 
| OutLine   | 轮廓信息；墙的顶面轮廓线   | [          [                {                            "X":-2376.32,                            "Y":6908.54,                            "Z":0                        },                        {                            "X":521.91,                            "Y":-4715.65,                            "Z":0                        }          ]      ]  数组的数组   | 
| Width   | 墙宽   | 120   | 



## 构件-虚拟墙
```json
{
    "Id":"10009",
    "ElementType":"VirtualWall",
    "SourceId":"1159597",
    "Name":"",
    "Location":{
        "Type":"Line",
        "Points":[
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    },
    "OutLine":[
        [
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    ]
}
```


| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10009   | 
| ElementType   | 构件类型，这里标识为虚拟墙   | VirtualWall   | 
| SourceId   | 对应BIM模型中的Id   | 1159597   | 
| Name   | 构件名称   |    | 
| Location   | 定位信息   | 线定位信息   | 
| OutLine   | 轮廓信息；和定位信息中点描述相同   | [          [              "-2090.61,10352.94,1128",              "-2090.61,-17461.31,1128"          ]      ]  数组的数组   | 

## 构件-门

```json
{
    "Id":"10009",
            "FamilyName":"单扇 - 与墙齐",
            "FamilySymbol":"0762 x 2032mm",
    "ElementType":"Door",
    "SourceId":"1159597",
    "Name":"双开门",
 "Location":{
                    "Type":"Point",
                    "Points":[
                        {
                            "X":5026.81,
                            "Y":7008.54,
                            "Z":0
                        }
                    ]
                },
                "OutLine":[
                    [
                        {
                            "X":4650.12,
                            "Y":7008.54,
                            "Z":0
                        },
                        {
                            "X":5401.81,
                            "Y":7008.54,
                            "Z":0
                        }
                    ]
                ],
    "Owner":"10008",
    "FaceDirection":"0,1,0",
    "HandDirection":"1,0,0",
    "Thick":120
}
```

| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10009   | 
| ElementType   | 构件类型，这里标识为门   | Door   | 
| SourceId   | 对应BIM模型中的Id   | 1159597   | 
| Name   | 构件名称   |    | 
| Location   | 定位信息   | 点定位   | 
| OutLine   | 轮廓信息；一段直线段   | ```json[          [                {                            "X":4650.12,                            "Y":7008.54,                            "Z":0                        },                        {                            "X":5401.81,                            "Y":7008.54,                            "Z":0                        }          ]      ] ''' 数组的数组   | 
| Owner   | 所属主体（墙），引用id   | 10008   | 
| FaceDirection   | 门开口方向   |  "0,1,0"  方向向量   | 
| HandDirection   |    |  "1,0,0"  方向向量   | 
| Thick   | 厚度   | 120   | 
| FamilyName   | 设备的族名称   | 单扇 - 与墙齐   | 
| FamilySymbol   | 设备的族类型   | 0762 x 2032mm   | 



## 构件-窗

```json
{
    "Id":"10009",
            "FamilyName":"固定",
            "FamilySymbol":"0406 x 1220mm",
    "ElementType":"Window",
    "SourceId":"1159597",
    "Name":"窗",
    "Location":{
        "Type":"Point",
        "Points":[
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    },
    "OutLine":[
        [
                {
                            "X":4650.12,
                            "Y":7008.54,
                            "Z":0
                        },
                        {
                            "X":5401.81,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    ],
    "Owner":"10008",
    "Thick":120
}
```


| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10009   | 
| ElementType   | 构件类型，这里标识为窗   | Window   | 
| SourceId   | 对应BIM模型中的Id   | 1159597   | 
| Name   | 构件名称   |    | 
| Location   | 定位信息   | 点定位   | 
| OutLine   | 轮廓信息；一段直线段   | [          [                 {                            "X":4650.12,                            "Y":7008.54,                            "Z":0                        },                        {                            "X":5401.81,                            "Y":7008.54,                            "Z":0                        }          ]      ]  数组的数组   | 
| Owner   | 所属主体（墙），引用id   | 10008   | 
| Thick   | 厚度   | 120   | 
| FamilyName   | 设备的族名称   | 固定   | 
| FamilySymbol   | 设备的族类型   | 0762 x 2032mm   | 


## 构件-柱子

```json
{
    "Id":"10009",
    "FamilyName":"矩形柱",
            "FamilySymbol":"457 x 475mm",
    "ElementType":"Column",
    "SourceId":"1159597",
    "Name":"200×200",
    "Location":{
        "Type":"Point",
        "Points":[
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    },
    "OutLine":[
        [
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    ],
  "RoomBoundary":true,
}
```


| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10009   | 
| ElementType   | 构件类型，这里标识为柱子   | Column   | 
| SourceId   | 对应BIM模型中的Id   | 1159597   | 
| Name   | 构件名称   |    | 
| Location   | 定位信息   | 点定位信息:```json{  "Type":"Point",              "Points":[                  {                            "X":-5498.19,                            "Y":7008.54,                            "Z":0                        }              ]  }```   | 
| OutLine   | 轮廓信息。模型顶面外轮廓   | [          [              "-2090.61,10352.94,1128",              "-2090.61,-17461.31,1128"          ]      ]  数组的数组   | 
| RoomBoundary   | 房间边界布尔类型   | true   | 
| FamilyName   | 设备的族名称   | 矩形柱   | 
| FamilySymbol   | 设备的族类型   | 457 x 475mm   | 


### 
### 
## 构件-空间
输入：Revit模型文件地址 *\F1.rvt
输出：
```json
{
    "Id":"10001",
    "ElementType":"Space",
    "SourceId":"316306",    
    "Height":3600,
    "Name":"空间 1",
    "Tag":"这里是一些补充信息",
    "Parameters":[
        {
            "Name":"编号",
            "Value":"1",
           
        },
        {
            "Name":"名称",
            "Value":"空间",
           
        }
    ],
    "Location":{
        "Type":"Point",
        "Points":[
            "13.9579280905739, 26.0937686739499, 0"
        ]
    },
    "OutLine":[
        [
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ],
        [
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    ],
    "BoundarySegments":[
        [
            "10001",
            "10002",
            "10003",
            "10004"
        ]
    ],
    "OutLine2":[
        [
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ],
        [
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    ]
}
```

| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10001   | 
| ElementType   | 构件类型，这里标识为一个空间   | Space   | 
| SourceId   | 对应BIM模型中的Id   | 316306   | 
| Height   | 空间的高度   | 3600   | 
| Name   | 构件名称   | 空间 1   | 
| Tag   | 这里是一些补充信息   |    | 
| Parameters   | 从BIM模型中读取到的参数   | 名称、编号   | 
| Location   | 定位信息   | 一般是定位点，也可以是定位线   | 
| BoundarySegments   | 关联轮廓信息   | 关联轮廓信息对应的id数组，数组的数组   | 
| OutLine   | 轮廓线   | 围成几何体的线，外圈是逆时针，内圈为顺时针   | 
| OutLine2   | 空间外轮廓   | 取关联墙的外面轮廓   | 

### 

### 

## 构件-设备
```json
{
            "Id":"10002",
            "ElementType":"Equipment",
            "SourceId":"320720",
            "Name":"CCOP-供冷冷却水泵",
            "Tag":"这里是一些补充信息，例如待建模传入的岗位Id",
            "FamilyName":"CCOP-供冷冷却水泵",
            "FamilySymbol":"0762 x 2032mm",
            "Parameters":[
                {
                    "Name":"设备本地名称",
                    "Value":"",
                   
                },
                {
                    "Name":"设备本地编码",
                    "Value":"",
                   
                },
                {
                    "Name":"PropertyID",
                    "Value":"Pe******",                   
                }
            ],
            "Location":{
                "Type":"Point",
                "Points":[
       {
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
                ]
            },
            "Rotation":0,
            "OutLine":[
                [
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
                ]
            ]
        }
```

| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10001   | 
| ElementType   | 构件类型，这里标识为一个空间   | Equipment   | 
| SourceId   | 对应BIM模型中的Id   | 316306   | 
| Name   | 构件名称，可能与族名称一样，但不等价于族名称，   | CCOP-供冷冷却水泵   | 
| Tag   | 这里是一些补充信息   |    | 
| FamilyName   | 设备的族名称   | CCOP-供冷冷却水泵   | 
| FamilySymbol   | 设备的族类型   | 0762 x 2032mm   | 
| Parameters   | 从BIM模型中读取到的参数   | 设备本地名称、设备本地编码编码、资产Id   | 
| Location   | 定位信息   | 一般是定位点，也可以是定位线   | 
| Rotation   | 旋转角度   | 0   | 
| OutLine   | 轮廓线   | 围成几何体的线，外圈是逆时针，内圈为顺时针   | 

## 构件-部件
```json
{
            "Id":"10001",
            "ElementType":"EquipPart",
            "SourceId":"333940",
            "Name":"配电抽屉320*800",
            "Tag":"这里是一些补充信息，例如待建模传入的岗位Id",
            "FamilyName":"TDLSDW-低压配电抽屉",
            "FamilySymbol":"0762 x 2032mm",
            "Owner":"10002",
            "Parameters":[
                {
                    "Name":"设备本地名称",
                    "Value":"",
                   
                },
                {
                    "Name":"设备本地编码",
                    "Value":"",                   
                },
                {
                    "Name":"PropertyID",
                    "Value":"Pe******",                   
                }
            ],
            "Location":{
                "Type":"Point",
                "Points":[
        {
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
                ]
            },
            "Rotation":0,
            "OutLine":[
                [
           {
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
                ]
            ]
        }
```

| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10001   | 
| ElementType   | 构件类型，这里标识为一个空间   | EquipPart   | 
| SourceId   | 对应BIM模型中的Id   | 316306   | 
| Name   | 构件名称，可能与族名称一样，但不等价于族名称，   | 配电抽屉320*800   | 
| Tag   | 这里是一些补充信息   |    | 
| FamilyName   | 设备的族名称   | TDLSDW-低压配电抽屉   | 
| FamilySymbol   | 设备的族类型   | 0762 x 2032mm   | 
| Owner   | 关系-所属设备的Id，这里是引用的Id,不是BIM模型中的Id   | 10002   | 
| Parameters   | 从BIM模型中读取到的参数   | 设备本地名称、设备本地编码编码、资产Id   | 
| Location   | 定位信息   | 一般是定位点，也可以是定位线   | 
| Rotation   | 旋转角度   | 0   | 
| OutLine   | 轮廓线   | 围成几何体的线，外圈是逆时针，内圈为顺时针   | 


## 构件-信标
暂时不清楚
# 
## 构件-水管    

```json
{
    "Id":"10009",
    "ElementType":"Pipe",
    "SourceId":"1159597",
    "Name":"铝塑复合管-卡套",
    "Location":{
        "Type":"Line",
        "Points":[
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    },
    "OutLine":[
        [
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    ],
    "MepSystemTypeName":"中水系统",
    "Diameter":80,
    "ConnectedIds":[
        "10010",
        "10011"
    ],
 "Parameters":[
                {
                    "Name":"注释",
                    "Value":"start",
                   
                }
            ]
}
```


| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10009   | 
| ElementType   | 构件类型，这里标识为水管   | Pipe   | 
| SourceId   | 对应BIM模型中的Id   | 1159597   | 
| Name   | 构件名称   |    | 
| Location   | 定位信息   | 是一个直线段:```json{  "Type":"Line",              "Points":[                  "-1997.74,-17554.18,1128",                  "29909.39,-17554.18,1128"              ]  }``` | 
| OutLine   | 轮廓信息   |```json [          [              "-2090.61,10352.94,1128",              "-2090.61,-17461.31,1128"          ]      ] ```数组的数组   | 
| MepSystemTypeName   | 管道系统类型名称   | 中水系统   | 
| Diameter   | 直径   | 80   | 
| ConnectedIds   | 包含的连接点信息。连接点id信息的数组   | :[          "10016"      ]   | 
| Parameters   | 从BIM模型中读取到的参数   | 注释   | 

## 构件-风管

```json
{
    "Id":"10009",
    "ElementType":"Duct",
    "SourceId":"1159597",
    "Name":"铝材",
    "Location":{
        "Type":"Line",
        "Points":[
    {
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    },
    "OutLine":[
        [
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    ],
    "MepSystemTypeName":"中水系统",
    "Shape":"Circle|Rectangle|Ellipse",
    "Diameter":13.34,
    "Width":200,
    "Height":100,
    "ConnectedIds":[
        "10010",
        "10011"
    ],
 "Parameters":[
                {
                    "Name":"注释",
                    "Value":"start",                   
                }
            ]
}
```

| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10009   | 
| ElementType   | 构件类型，这里标识为风管   | Duct   | 
| SourceId   | 对应BIM模型中的Id   | 1159597   | 
| Name   | 构件名称   |    | 
| Location   | 道定位信息   | 是一个直线段:{  "Type":"Line",              "Points":[                  "-1997.74,-17554.18,1128",                  "29909.39,-17554.18,1128"              ]  }v   | 
| OutLine   | 轮廓信息   | [          [              "-2090.61,10352.94,1128",              "-2090.61,-17461.31,1128"          ]      ]  数组的数组   | 
| MepSystemTypeName   | 管道系统类型名称   | 送风   | 
| Shape   | 风管形状；枚举类型   | Circle\|Rectangle\|Ellipse   | 
| Diameter   | 直径径；当形状为Circle时有效   | 80   | 
| Width   | 宽度；当形状为Rectangle或者Ellipse   | 300   | 
| Height   | 高度；当形状为Rectangle或者Ellipse   | 100   | 
| ConnectedIds   | 包含的连接点信息。连接点id信息的数组   | :[          "10016"    "10017"      ]   | 
| Parameters   | 从BIM模型中读取到的参数   | 注释   | 


## 其他连接构件（有Connector的,管道和设备以外的构件）

```json
{
    "Id":"10009",
            "FamilyName":"T 形三通 - 常规",
            "FamilySymbol":"标准",
    "ElementType":"JoinObject",
    "SourceId":"1159597",
    "Name":"标准",
    "Location":{
        "Type":"Point",
        "Points":[
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    },
    "OutLine":[
        [
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    ],
            "Rotation":0,
    "ConnectedIds":[
        "10010",
        "10011"
    ]
}
```

| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10009   | 
| ElementType   | 构件类型，这里标识其他构件   | JoinObject   | 
| SourceId   | 对应BIM模型中的Id   | 1159597   | 
| Name   | 构件名称   |    | 
| Location   | 定位信息   | 点定位信息和线定位信息都有可能存在   | 
| OutLine   | 轮廓信息   | [          [              "-2090.61,10352.94,1128",              "-2090.61,-17461.31,1128"          ]      ]  数组的数组   | 
| ConnectedIds   | 包含的连接点信息。连接点id信息的数组   | :[          "10016"      ]   | 
| FamilyName   | 设备的族名称   | T 形三通 - 常规   | 
| Rotation   | 旋转角度   | 0   | 
| FamilySymbol   | 设备的族类型   | 标准   | 



## 其他构件（除门、窗、柱、设备、部件、其他连接构件以外的构件）

```json
{
    "Id":"10009",
            "FamilyName":"T 形三通 - 常规",
            "FamilySymbol":"标准",
    "ElementType":"Other",
    "SourceId":"1159597",
    "Name":"标准",
    "Location":{
        "Type":"Point",
        "Points":[
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    },
            "Rotation":0,
    "OutLine":[
        [
{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
        ]
    ]
}
```

| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10009   | 
| FamilyName   | 设备的族名称   | T 形三通 - 常规   | 
| FamilySymbol   | 设备的族类型   | 标准   | 
| ElementType   | 构件类型，这里标识其他构件   | Other   | 
| SourceId   | 对应BIM模型中的Id   | 1159597   | 
| Name   | 构件名称   |    | 
| Location   | 定位信息   | 点定位信息和线定位信息都有可能存在   | 
| OutLine   | 轮廓信息   | [          [              "-2090.61,10352.94,1128",              "-2090.61,-17461.31,1128"          ]      ]  数组的数组   | 
| Rotation   | 旋转角度   | 0   | 


### 连接点 connecor
```json
{
    "Id":"10010",
    "ElementType":"Connector",
    "SourceId":"1159597_C_0",
    "Domain":"DomainPiping",
    "MepSystemTypeName":"中水系统",
    "IsConnected":true,
    "Origin":{
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        },
    "Owner":"10009",
    "ConnectedIds":[
        "10016"
    ]
}
```


| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10001   | 
| ElementType   | 构件类型，这里标识为连接件   | Connector   | 
| SourceId   | 对应BIM模型中的Id   | 316306   | 
| Domain   | 连接点所属专业类别   | DomainPiping   | 
| Description   | 连接点描述信息   |    | 
| MepSystemTypeName   | 连接点所属管网的系统类型名称   | 中水系统   | 
| IsConnected   | 标志是否处于连接状态   | 类型为布尔类型   | 
| Origin   | 连接点定位信息   | XYZ形式坐标   | 
| Owner   | 连接点所关联的父对象引用Id   | 10009;关联id为10009的实体对象   | 
| ConnectedIds   |   和连接点存在连接关系的其他连接点的id信息数组   | :[          "10016"      ]   | 



### 空间轮廓线段
```json
{
    "Id":"10009",
    "ElementType":"BoundarySegment",
    "Reference":"10556",
    "Curve":[
      {
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        }
    ]
}
```

| 名称   | 解释   | 示例    
|:----|:----|:----|
| Id   | 在提取过程中生成的Id   | 10009   | 
| ElementType   | 构件类型，这里标识为 空间轮廓线段   | BoundarySegment   | 
| Reference   | 线段关联实体构件Id,可以为空, 指向墙id, 虚拟墙id, 柱子id(可能门窗id)   | 10556   | 
| Curve   | 线,点的数组   |         [              "-2090.61,10352.94,1128",              "-2090.61,-17461.31,1128"          ]      | 


### 
### 定位信息
输入：
输出：
```json
{
                    "Type":"Line",
                    "Points":[
                        {
                            "X":-5498.19,
                            "Y":7008.54,
                            "Z":0
                        },
                        {
                            "X":12101.81,
                            "Y":7008.54,
                            "Z":0
                        }
                    ]
                }
```

| 名称   | 解释   | 示例    
|:----|:----|:----|
| Type   | 定位信息的类型   | Point,Line,   | 
| Points   | 定位点的集合，如果为弧被拆分成多段线表示。   |    | 

###  
## 总体返回对象信息
地址：[http://git.sagacloud.cn/revit/RevitExport/src/4246dff16d3268ae165222687fa6b8a70779d4f0/20190628114542.json](http://git.sagacloud.cn/revit/RevitExport/src/4246dff16d3268ae165222687fa6b8a70779d4f0/20190628114542.json)
